<?php
/**
 * Created by Proxy-Spider http://proxy-spider.com
 * Version 0.1
 */

namespace ProxySpiderApi;

/**
 * Class ProxySpiderApi
 *
 * @package ProxySpiderApi
 */
class ProxySpiderApi
{
    /**
     * System constants
     */
    const ENDPOINT = 'http://proxy-spider.com/api/proxies.json';

    /**
     * Order constants
     */
    const ORDER_BEST    = 'best';
    const ORDER_NEW     = 'new';
    const ORDER_RANDOM  = 'random';

    /**
     * Response time constants
     */
    const RESPONSE_TIME_SLOW    = 'slow';
    const RESPONSE_TIME_MEDIUM  = 'medium';
    const RESPONSE_TIME_FAST    = 'fast';

    /**
     * Proxy type constants
     */
    const PROXY_TYPE_ELITE       = 'elite';
    const PROXY_TYPE_ANONYMOUS   = 'anonymous';
    const PROXY_TYPE_TRANSPARENT = 'transparent';

    /**
     * Protocol constants
     */
    const PROTOCOL_HTTP   = 'http';
    const PROTOCOL_HTTPS  = 'https';
    const PROTOCOL_SOCKS4 = 'socks4';
    const PROTOCOL_SOCKS5 = 'socks5';

    /**
     * Support constants
     */
    const SUPPORT_GET       = 'get';
    const SUPPORT_POST      = 'post';
    const SUPPORT_COOKIE    = 'cookie';
    const SUPPORT_REFERER   = 'referer';
    const SUPPORT_USERAGENT = 'useragent';

    /**
     * @var string
     */
    protected $apiKey = 'YOUR API KEY';

    /**
     * @var int
     */
    protected $limit = 1;

    /**
     * @var int
     */
    protected $page = 1;

    /**
     * @var int
     */
    protected $test = 1;

    /**
     * @var string
     */
    protected $order = ProxySpiderApi::ORDER_BEST;

    /**
     * @var array
     */
    protected $responseTime = array();

    /**
     * @var array
     */
    protected $proxyType = array();

    /**
     * @var array
     */
    protected $protocol = array();

    /**
     * @var array
     */
    protected $support = array();

    /**
     * @var array
     */
    protected $countryCode = array();

    /**
     * @return ProxySpiderApiResponse
     */
    public function callApi()
    {
        $content = ProxySpiderApiLoader::getContent($this->createRequest());
        $response = new ProxySpiderApiResponse($content);

        return $response;
    }

    /**
     * @return string
     */
    public function getRequest()
    {
        return $this->createRequest();
    }

    /**
     * @return string
     */
    protected function createRequest()
    {
        return ProxySpiderApi::ENDPOINT
            . '?api_key='       . $this->getApiKey()
            . '&limit='         . $this->getLimit()
            . '&page='          . $this->getPage()
            . '&order='         . $this->getOrder()
            . '&country_code='  . $this->getCountryCode()
            . '&response_time=' . $this->getResponseTime()
            . '&type='          . $this->getProxyType()
            . '&protocols='     . $this->getProtocol()
            . '&supports='      . $this->getSupport()
            . '&test='          . $this->getTest();
    }

    /**
     * @return string
     */
    public function getApiKey()
    {
        return $this->apiKey;
    }

    /**
     * @param string $apiKey
     * @return ProxySpiderApi
     */
    public function setApiKey($apiKey)
    {
        $this->apiKey = $apiKey;

        return $this;
    }

    /**
     * @return int
     */
    public function getLimit()
    {
        return $this->limit;
    }

    /**
     * @param int $limit
     * @return ProxySpiderApi
     */
    public function setLimit($limit)
    {
        $this->limit = ($limit <= 0) ? 1 : $limit;
        $this->limit = ($this->limit > 1000) ? 1000 : $this->limit;

        return $this;
    }

    /**
     * @return int
     */
    public function getPage()
    {
        return $this->page;
    }

    /**
     * @param int $page
     * @return ProxySpiderApi
     */
    public function setPage($page)
    {
        $this->page = ($page <= 0) ? 1 : $page;

        return $this;
    }

    /**
     * @return string
     */
    public function getOrder()
    {
        return $this->order;
    }

    /**
     * @param string $order
     * @return ProxySpiderApi
     */
    public function setOrder($order)
    {
        $this->order = (in_array($order, array(
            ProxySpiderApi::ORDER_BEST,
            ProxySpiderApi::ORDER_NEW,
            ProxySpiderApi::ORDER_RANDOM
        ))) ? $order : ProxySpiderApi::ORDER_BEST;

        return $this;
    }

    /**
     * @return int
     */
    public function getTest()
    {
        return $this->test;
    }

    /**
     * @param int $test
     * @return ProxySpiderApi
     */
    public function setTest($test)
    {
        $this->test = ((int) $test === 1) ? 1 : 0;

        return $this;
    }

    /**
     * @return string
     */
    public function getResponseTime()
    {
        return implode(',', $this->responseTime);
    }

    /**
     * @param array $responseTime
     * @return ProxySpiderApi
     */
    public function setResponseTime($responseTime)
    {
        if (is_array($responseTime)) {
            $this->responseTime = array();
            foreach ($responseTime as $value) {
                if (in_array($value, array(
                    ProxySpiderApi::RESPONSE_TIME_FAST,
                    ProxySpiderApi::RESPONSE_TIME_MEDIUM,
                    ProxySpiderApi::RESPONSE_TIME_SLOW,
                ))) {
                    $this->responseTime[] = $value;
                }
            }
        } else {
            $this->responseTime = (in_array($responseTime, array(
                ProxySpiderApi::RESPONSE_TIME_FAST,
                ProxySpiderApi::RESPONSE_TIME_MEDIUM,
                ProxySpiderApi::RESPONSE_TIME_SLOW,
            ))) ? array($responseTime) : array();
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getProxyType()
    {
        return implode(',', $this->proxyType);
    }

    /**
     * @param array $proxyType
     * @return ProxySpiderApi
     */
    public function setProxyType($proxyType)
    {
        if (is_array($proxyType)) {
            $this->proxyType = array();
            foreach ($proxyType as $value) {
                if (in_array($value, array(
                    ProxySpiderApi::PROXY_TYPE_ELITE,
                    ProxySpiderApi::PROXY_TYPE_ANONYMOUS,
                    ProxySpiderApi::PROXY_TYPE_TRANSPARENT,
                ))) {
                    $this->proxyType[] = $value;
                }
            }
        } else {
            $this->proxyType = (in_array($proxyType, array(
                ProxySpiderApi::PROXY_TYPE_ELITE,
                ProxySpiderApi::PROXY_TYPE_ANONYMOUS,
                ProxySpiderApi::PROXY_TYPE_TRANSPARENT,
            ))) ? array($proxyType) : array();
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getProtocol()
    {
        return implode(',', $this->protocol);
    }

    /**
     * @param array $protocol
     * @return ProxySpiderApi
     */
    public function setProtocol($protocol)
    {
        if (is_array($protocol)) {
            $this->protocol = array();
            foreach ($protocol as $value) {
                if (in_array($value, array(
                    ProxySpiderApi::PROTOCOL_HTTP,
                    ProxySpiderApi::PROTOCOL_HTTPS,
                    ProxySpiderApi::PROTOCOL_SOCKS4,
                    ProxySpiderApi::PROTOCOL_SOCKS5,
                ))) {
                    $this->protocol[] = $value;
                }
            }
        } else {
            $this->protocol = (in_array($protocol, array(
                ProxySpiderApi::PROTOCOL_HTTP,
                ProxySpiderApi::PROTOCOL_HTTPS,
                ProxySpiderApi::PROTOCOL_SOCKS4,
                ProxySpiderApi::PROTOCOL_SOCKS5,
            ))) ? array($protocol) : array();
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getSupport()
    {
        return implode(',', $this->support);
    }

    /**
     * @param array $support
     * @return ProxySpiderApi
     */
    public function setSupport($support)
    {
        if (is_array($support)) {
            $this->support = array();
            foreach ($support as $value) {
                if (in_array($value, array(
                    ProxySpiderApi::SUPPORT_GET,
                    ProxySpiderApi::SUPPORT_POST,
                    ProxySpiderApi::SUPPORT_COOKIE,
                    ProxySpiderApi::SUPPORT_REFERER,
                    ProxySpiderApi::SUPPORT_USERAGENT,
                ))) {
                    $this->support[] = $value;
                }
            }
        } else {
            $this->support = (in_array($support, array(
                ProxySpiderApi::SUPPORT_GET,
                ProxySpiderApi::SUPPORT_POST,
                ProxySpiderApi::SUPPORT_COOKIE,
                ProxySpiderApi::SUPPORT_REFERER,
                ProxySpiderApi::SUPPORT_USERAGENT,
            ))) ? array($support) : array();
        }

        return $this;
    }

    /**
     * @return string
     */
    public function getCountryCode()
    {
        return implode(',', $this->countryCode);
    }

    /**
     * @param array $countryCode
     * @return ProxySpiderApi
     */
    public function setCountryCode($countryCode)
    {
        if (is_array($countryCode)) {
            $this->countryCode = $countryCode;
        } else {
            $this->countryCode = array($countryCode);
        }

        return $this;
    }

}